"{ Package: 'stx:libscm/common' }"

AbstractSourceCodeManager subclass:#SCMAbstractSourceCodeManager
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Common-StX'
!

SCMAbstractSourceCodeManager class instanceVariableNames:'registry uuid'

"
 No other class instance variables are inherited by this class.
"
!

!SCMAbstractSourceCodeManager class methodsFor:'documentation'!

documentation
"
    A base abstract class for all stx:libscm based source code managers

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!SCMAbstractSourceCodeManager class methodsFor:'* As yet uncategorized *'!

savePreferencesOn:aFileStream

    "Created: / 18-10-2012 / 15:53:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMAbstractSourceCodeManager class methodsFor:'accessing'!

repositoryNameForPackage:arg
    "raise an error: must be redefined in concrete subclass(es)"

    ^ self subclassResponsibility
!

temporaryWorkingCopyDirectory
    "Return a directory for temporary working copies"

    ^ UserPreferences current 
        at:(self nameWithoutPrefix,'.temporaryWCDirectory') 
        ifAbsent: [ Filename tempDirectory ]

    "Created: / 14-11-2012 / 22:54:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

temporaryWorkingCopyDirectory: directory
    "Sets a directory for temporary working copies"

    ^ UserPreferences current 
        at:(self nameWithoutPrefix,'.temporaryWCDirectory') 
        put: directory asString

    "Created: / 14-11-2012 / 23:24:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

utilities

    ^SCMCommonSourceCodeManagerUtilities forManager: self.

    "Created: / 06-10-2012 / 18:16:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 14-11-2012 / 22:49:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMAbstractSourceCodeManager class methodsFor:'accessing-classes'!

commitDialogClass
    "Answer a dialog class to be used for commits"

    ^ self subclassResponsibility

    "Modified (comment): / 14-11-2012 / 00:56:39 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

packageRegistryClass
    "Answer the package manager class used to get
     package models"

    ^ self subclassResponsibility

    "Modified (comment): / 14-11-2012 / 00:57:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMAbstractSourceCodeManager class methodsFor:'basic access'!

checkinClass:aClass fileName:classFileName directory:packageDir module:moduleDir source:sourceFile logMessage:logMessage force:force
    "checkin of a class into the source repository.
     Return true if ok, false if not."

    ^ self shouldImplement
!

checkoutModule:aModule directory:aPackage andDo:aBlock
    "check out everything from a package into a temporary directory.
     Then evaluate aBlock, passing the name of that temp-directory.
     Afterwards, the tempDir is removed.
     Return true, if OK, false if any error occurred."

    ^ self shouldImplement
!

streamForClass:aClass fileName:classFileName revision:revision directory:packageDir module:moduleDir cache:doCache
    "extract a classes source code and return an open readStream on it.
     A revision of nil selects the current (in image) revision.
     The classes source code is extracted using the revision and the sourceCodeInfo,
     which itself is extracted from the classes packageString."

    ^ self shouldImplement
! !

!SCMAbstractSourceCodeManager class methodsFor:'basic administration'!

checkForExistingContainer:fileName inModule:moduleName directory:dirName
    "check for a container to be present"

    ^ self shouldImplement
!

checkForExistingModule:moduleName
    "check for a module directory to be present"

    ^ self shouldImplement
!

checkForExistingModule:moduleDir directory:packageDir
    "check for a package directory to be present"

    ^ self shouldImplement
!

createContainerFor:aClass inModule:moduleName directory:dirName container:fileName
    "create a new container & check into it an initial version of aClass"

    ^ self shouldImplement
!

createModule:moduleName
    "create a new module directory"

    ^ self shouldImplement
!

createModule:module directory:directory
    "create a new package directory"

    ^ self shouldImplement
!

initialRevisionStringFor:aClass inModule:moduleDir directory:packageDir container:fileName
    "return a string usable as initial revision string"

    ^ self shouldImplement
!

revisionLogOf:clsOrNil fromRevision:rev1OrNil toRevision:rev2OrNil numberOfRevisions:limitOrNil fileName:classFileName directory:packageDir module:moduleDir
    "actually do return a revisionLog. The main worker method.
     This must be implemented by a concrete source-code manager"

    ^ self shouldImplement
! !

!SCMAbstractSourceCodeManager class methodsFor:'queries'!

isResponsibleForPackage:arg
    "raise an error: must be redefined in concrete subclass(es)"

    ^ self subclassResponsibility
! !

!SCMAbstractSourceCodeManager class methodsFor:'source code administration'!

getExistingContainersInModule:aModule directory:aPackage
    "{ Pragma: +optSpace }"

    ^ self shouldImplement
!

getExistingDirectoriesInModule:aModule
    "{ Pragma: +optSpace }"

    ^ self shouldImplement
!

getExistingModules
    "{ Pragma: +optSpace }"

    ^ self shouldImplement
!

removeContainer:container inModule:module directory:directory
    "remove a container"

    ^ self shouldImplement
!

revisionInfoFromString:aString
    "{ Pragma: +optSpace }"

    ^ nil

    "Modified: / 06-10-2012 / 23:58:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMAbstractSourceCodeManager class methodsFor:'subclass responsibility'!

reportHistoryLogSince:timeGoal filterSTSources:filter filterUser:userFilter filterRepository:repositoryFilter filterModules:moduleFilter inTo:aBlock
    "superclass AbstractSourceCodeManager class says that I am responsible to implement this method"

    ^ self shouldImplement
! !

!SCMAbstractSourceCodeManager class methodsFor:'testing'!

isAbstract
    ^ self == SCMAbstractSourceCodeManager

    "Created: / 13-11-2012 / 23:07:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMAbstractSourceCodeManager class methodsFor:'documentation'!

version_HG
    "Never, ever change this method. Ask JV or CG why"
    ^thisContext method mclass theNonMetaclass instVarNamed: #revision
!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
